/*
Copyright (C) 2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.guicomponents;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JPanel;

public final class JythonPanel extends JPanel {
    private static final long serialVersionUID = 1L;

    private String pythonDir;
    private GuiUtils guiUtils;
    private transient JythonEngine jythonEngine = null;

    private static class PythonExtensionFilter implements FilenameFilter {

        @Override
        public boolean accept(File dir, String name) {
            return name.endsWith(".py");
        }

    }

    /**
     * Creates new form JythonPanel
     */
    public JythonPanel() {
        initComponents();
        //setPythonDir("python");
    }

    public JythonPanel(String pythonDir, GuiUtils guiUtils) {
        this();
        this.guiUtils = guiUtils;
        setPythonDir(pythonDir);
    }

    public void setGuiUtils(GuiUtils guiUtils) {
        this.guiUtils = guiUtils;
    }

    public String getPythonDir() {
        return pythonDir;
    }

    public void setPythonDir(String pythonDir) {
        if (pythonDir == null)
            return;
        File f = new File(pythonDir);
        if (!f.isDirectory()) {
            if (guiUtils != null)
                guiUtils.error(pythonDir + " is not a directory");
            else
                System.err.println(pythonDir + " is not a directory");
            return;
        }

        this.pythonDir = pythonDir;
        String[] files = f.list();
        Vector<String> subdirs = new Vector<String>();
        for (String s : files) {
            File file = new File(f, s);
            if (file.isDirectory())
                subdirs.add(s);
        }

        subdirComboBox.setModel(new DefaultComboBoxModel(subdirs));
        subdirComboBoxActionPerformed(null);
    }

    private File getDirectory() {
        return new File(pythonDir, (String) subdirComboBox.getSelectedItem());
    }

    private File getFile() {
        return new File(getDirectory(), (String) fileComboBox.getSelectedItem());
    }

    public boolean init(String initfile) {
        String runfile = (new File(initfile)).isAbsolute() ? initfile : (new File(pythonDir, initfile)).getAbsolutePath();
        jythonEngine = new JythonEngine(runfile);
        return true;
    }

    public void assign(String pythonName, Object object) {
        jythonEngine.set(pythonName, object);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        subdirComboBox = new javax.swing.JComboBox();
        fileComboBox = new javax.swing.JComboBox();
        executeFileButton = new javax.swing.JButton();
        editButton = new javax.swing.JButton();
        jComboBox1 = new javax.swing.JComboBox();
        jButton1 = new javax.swing.JButton();
        pythonCommandTextField = new javax.swing.JTextField();

        subdirComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        subdirComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                subdirComboBoxActionPerformed(evt);
            }
        });

        fileComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        executeFileButton.setText("Execute");
        executeFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                executeFileButtonActionPerformed(evt);
            }
        });

        editButton.setText("Edit...");
        editButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editButtonActionPerformed(evt);
            }
        });

        jComboBox1.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        jButton1.setText("Run");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        pythonCommandTextField.setText("jTextField1");
        pythonCommandTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pythonCommandTextFieldActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jComboBox1, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(subdirComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 125, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(fileComboBox, 0, 191, Short.MAX_VALUE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(executeFileButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(editButton))
                    .addComponent(jButton1))
                .addGap(32, 32, 32))
            .addComponent(pythonCommandTextField)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(7, 7, 7)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(subdirComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(fileComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(executeFileButton)
                    .addComponent(editButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pythonCommandTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(35, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void subdirComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_subdirComboBoxActionPerformed
        String[] candidates = getDirectory().list(new PythonExtensionFilter());
        Vector<String> files = new Vector<String>();
        for (String s : candidates) {
            File file = new File(getDirectory(), s);
            if (file.isFile())
                files.add(s);
        }
        fileComboBox.setModel(new DefaultComboBoxModel(files));
    }//GEN-LAST:event_subdirComboBoxActionPerformed

    private void executeFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_executeFileButtonActionPerformed
        if (jythonEngine == null) {
            guiUtils.error("Jythonengine not initialized");
            return;
        }
        jythonEngine.execFile(getFile().getPath());
    }//GEN-LAST:event_executeFileButtonActionPerformed

    private void editButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editButtonActionPerformed
        guiUtils.edit(getFile());
    }//GEN-LAST:event_editButtonActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jButton1ActionPerformed

    private void pythonCommandTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pythonCommandTextFieldActionPerformed
        jythonEngine.exec(pythonCommandTextField.getText());
    }//GEN-LAST:event_pythonCommandTextFieldActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton editButton;
    private javax.swing.JButton executeFileButton;
    private javax.swing.JComboBox fileComboBox;
    private javax.swing.JButton jButton1;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JTextField pythonCommandTextField;
    private javax.swing.JComboBox subdirComboBox;
    // End of variables declaration//GEN-END:variables
}
